/****************************************************************************
** Meta object code from reading C++ file 'uniformmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtquickeffectmaker/tools/qqem/uniformmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'uniformmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12UniformModelE_t {};
} // unnamed namespace

template <> constexpr inline auto UniformModel::qt_create_metaobjectdata<qt_meta_tag_ZN12UniformModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "UniformModel",
        "uniformsChanged",
        "",
        "qmlComponentChanged",
        "addFSCode",
        "code",
        "updateRow",
        "nodeId",
        "rowIndex",
        "type",
        "id",
        "QVariant",
        "defaultValue",
        "description",
        "customValue",
        "useCustomValue",
        "minValue",
        "maxValue",
        "enableMipmap",
        "exportImage",
        "removeRow",
        "rows",
        "resetValue",
        "setImage",
        "value",
        "moveIndex",
        "direction"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'uniformsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'qmlComponentChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addFSCode'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Method 'updateRow'
        QtMocHelpers::MethodData<bool(int, int, int, const QString &, const QVariant &, const QString &, const QString &, bool, const QVariant &, const QVariant &, bool, bool)>(6, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::Int, 9 }, { QMetaType::QString, 10 },
            { 0x80000000 | 11, 12 }, { QMetaType::QString, 13 }, { QMetaType::QString, 14 }, { QMetaType::Bool, 15 },
            { 0x80000000 | 11, 16 }, { 0x80000000 | 11, 17 }, { QMetaType::Bool, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Method 'removeRow'
        QtMocHelpers::MethodData<void(int, int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Int, 21 },
        }}),
        // Method 'removeRow'
        QtMocHelpers::MethodData<void(int)>(20, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Method 'resetValue'
        QtMocHelpers::MethodData<bool(int)>(22, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 8 },
        }}),
        // Method 'setImage'
        QtMocHelpers::MethodData<bool(int, const QVariant &)>(23, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 8 }, { 0x80000000 | 11, 24 },
        }}),
        // Method 'moveIndex'
        QtMocHelpers::MethodData<void(int, int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Int, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UniformModel, qt_meta_tag_ZN12UniformModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject UniformModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12UniformModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12UniformModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12UniformModelE_t>.metaTypes,
    nullptr
} };

void UniformModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UniformModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->uniformsChanged(); break;
        case 1: _t->qmlComponentChanged(); break;
        case 2: _t->addFSCode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: { bool _r = _t->updateRow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[12])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->removeRow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->removeRow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: { bool _r = _t->resetValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->setImage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->moveIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UniformModel::*)()>(_a, &UniformModel::uniformsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UniformModel::*)()>(_a, &UniformModel::qmlComponentChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UniformModel::*)(const QString & )>(_a, &UniformModel::addFSCode, 2))
            return;
    }
}

const QMetaObject *UniformModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UniformModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12UniformModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int UniformModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void UniformModel::uniformsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void UniformModel::qmlComponentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void UniformModel::addFSCode(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
