/****************************************************************************
** Meta object code from reading C++ file 'nodeview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtquickeffectmaker/tools/qqem/nodeview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'nodeview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8NodeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto NodeView::qt_create_metaobjectdata<qt_meta_tag_ZN8NodeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NodeView",
        "QML.Element",
        "NodeViewItem",
        "nodesModelChanged",
        "",
        "arrowsModelChanged",
        "activeArrowStartPointChanged",
        "activeArrowEndPointChanged",
        "activeArrowEnabledChanged",
        "activeNodesListChanged",
        "selectedNodeNameChanged",
        "selectedNodeDescriptionChanged",
        "selectedNodeFragmentCodeChanged",
        "selectedNodeVertexCodeChanged",
        "selectedNodeQmlCodeChanged",
        "nodeGraphCompleteChanged",
        "effectNodeSelectedChanged",
        "mainNodeSelectedChanged",
        "selectedNodeIdChanged",
        "codeSelectorModelChanged",
        "codeSelectorIndexChanged",
        "doubleClickNode",
        "openDeleteNodeDialog",
        "openNodeContextMenu",
        "updateActiveNodesList",
        "deleteSelectedNodes",
        "layoutNodes",
        "distribute",
        "selectSingleNode",
        "nodeId",
        "selectMainNode",
        "getNodeIdWithName",
        "name",
        "toggleNodeDisabled",
        "nodesModel",
        "NodesModel*",
        "arrowsModel",
        "ArrowsModel*",
        "activeArrowStartPoint",
        "QPointF",
        "activeArrowEndPoint",
        "activeArrowEnabled",
        "nodeGraphComplete",
        "effectNodeSelected",
        "mainNodeSelected",
        "selectedNodeId",
        "selectedNodeName",
        "selectedNodeDescription",
        "selectedNodeFragmentCode",
        "selectedNodeVertexCode",
        "selectedNodeQmlCode",
        "codeSelectorModel",
        "codeSelectorIndex"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'nodesModelChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'arrowsModelChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeArrowStartPointChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeArrowEndPointChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeArrowEnabledChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeNodesListChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedNodeNameChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedNodeDescriptionChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedNodeFragmentCodeChanged'
        QtMocHelpers::SignalData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedNodeVertexCodeChanged'
        QtMocHelpers::SignalData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedNodeQmlCodeChanged'
        QtMocHelpers::SignalData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nodeGraphCompleteChanged'
        QtMocHelpers::SignalData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'effectNodeSelectedChanged'
        QtMocHelpers::SignalData<void()>(16, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mainNodeSelectedChanged'
        QtMocHelpers::SignalData<void()>(17, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedNodeIdChanged'
        QtMocHelpers::SignalData<void()>(18, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'codeSelectorModelChanged'
        QtMocHelpers::SignalData<void()>(19, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'codeSelectorIndexChanged'
        QtMocHelpers::SignalData<void()>(20, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'doubleClickNode'
        QtMocHelpers::SignalData<void()>(21, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openDeleteNodeDialog'
        QtMocHelpers::SignalData<void()>(22, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openNodeContextMenu'
        QtMocHelpers::SignalData<void()>(23, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateActiveNodesList'
        QtMocHelpers::SlotData<void()>(24, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteSelectedNodes'
        QtMocHelpers::SlotData<void()>(25, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'layoutNodes'
        QtMocHelpers::SlotData<void(bool)>(26, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'selectSingleNode'
        QtMocHelpers::SlotData<void(int)>(28, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 },
        }}),
        // Slot 'selectMainNode'
        QtMocHelpers::SlotData<void()>(30, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getNodeIdWithName'
        QtMocHelpers::SlotData<int(const QString &)>(31, 4, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 32 },
        }}),
        // Slot 'toggleNodeDisabled'
        QtMocHelpers::SlotData<void()>(33, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'nodesModel'
        QtMocHelpers::PropertyData<NodesModel*>(34, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'arrowsModel'
        QtMocHelpers::PropertyData<ArrowsModel*>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'activeArrowStartPoint'
        QtMocHelpers::PropertyData<QPointF>(38, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'activeArrowEndPoint'
        QtMocHelpers::PropertyData<QPointF>(40, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'activeArrowEnabled'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'nodeGraphComplete'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags, 11),
        // property 'effectNodeSelected'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags, 12),
        // property 'mainNodeSelected'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags, 13),
        // property 'selectedNodeId'
        QtMocHelpers::PropertyData<int>(45, QMetaType::Int, QMC::DefaultPropertyFlags, 14),
        // property 'selectedNodeName'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'selectedNodeDescription'
        QtMocHelpers::PropertyData<QString>(47, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'selectedNodeFragmentCode'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'selectedNodeVertexCode'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'selectedNodeQmlCode'
        QtMocHelpers::PropertyData<QString>(50, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'codeSelectorModel'
        QtMocHelpers::PropertyData<QStringList>(51, QMetaType::QStringList, QMC::DefaultPropertyFlags, 15),
        // property 'codeSelectorIndex'
        QtMocHelpers::PropertyData<int>(52, QMetaType::Int, QMC::DefaultPropertyFlags, 16),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<NodeView, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject NodeView::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8NodeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8NodeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8NodeViewE_t>.metaTypes,
    nullptr
} };

void NodeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NodeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->nodesModelChanged(); break;
        case 1: _t->arrowsModelChanged(); break;
        case 2: _t->activeArrowStartPointChanged(); break;
        case 3: _t->activeArrowEndPointChanged(); break;
        case 4: _t->activeArrowEnabledChanged(); break;
        case 5: _t->activeNodesListChanged(); break;
        case 6: _t->selectedNodeNameChanged(); break;
        case 7: _t->selectedNodeDescriptionChanged(); break;
        case 8: _t->selectedNodeFragmentCodeChanged(); break;
        case 9: _t->selectedNodeVertexCodeChanged(); break;
        case 10: _t->selectedNodeQmlCodeChanged(); break;
        case 11: _t->nodeGraphCompleteChanged(); break;
        case 12: _t->effectNodeSelectedChanged(); break;
        case 13: _t->mainNodeSelectedChanged(); break;
        case 14: _t->selectedNodeIdChanged(); break;
        case 15: _t->codeSelectorModelChanged(); break;
        case 16: _t->codeSelectorIndexChanged(); break;
        case 17: _t->doubleClickNode(); break;
        case 18: _t->openDeleteNodeDialog(); break;
        case 19: _t->openNodeContextMenu(); break;
        case 20: _t->updateActiveNodesList(); break;
        case 21: _t->deleteSelectedNodes(); break;
        case 22: _t->layoutNodes((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->selectSingleNode((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->selectMainNode(); break;
        case 25: { int _r = _t->getNodeIdWithName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->toggleNodeDisabled(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::nodesModelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::arrowsModelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::activeArrowStartPointChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::activeArrowEndPointChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::activeArrowEnabledChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::activeNodesListChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::selectedNodeNameChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::selectedNodeDescriptionChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::selectedNodeFragmentCodeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::selectedNodeVertexCodeChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::selectedNodeQmlCodeChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::nodeGraphCompleteChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::effectNodeSelectedChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::mainNodeSelectedChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::selectedNodeIdChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::codeSelectorModelChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::codeSelectorIndexChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::doubleClickNode, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::openDeleteNodeDialog, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (NodeView::*)()>(_a, &NodeView::openNodeContextMenu, 19))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ArrowsModel* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< NodesModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<NodesModel**>(_v) = _t->nodesModel(); break;
        case 1: *reinterpret_cast<ArrowsModel**>(_v) = _t->arrowsModel(); break;
        case 2: *reinterpret_cast<QPointF*>(_v) = _t->activeArrowStartPoint(); break;
        case 3: *reinterpret_cast<QPointF*>(_v) = _t->activeArrowEndPoint(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->activeArrowEnabled(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->nodeGraphComplete(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->effectNodeSelected(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->mainNodeSelected(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->selectedNodeId(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->selectedNodeName(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->selectedNodeDescription(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->selectedNodeFragmentCode(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->selectedNodeVertexCode(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->selectedNodeQmlCode(); break;
        case 14: *reinterpret_cast<QStringList*>(_v) = _t->codeSelectorModel(); break;
        case 15: *reinterpret_cast<int*>(_v) = _t->codeSelectorIndex(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 9: _t->setSelectedNodeName(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setSelectedNodeDescription(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->setSelectedNodeFragmentCode(*reinterpret_cast<QString*>(_v)); break;
        case 12: _t->setSelectedNodeVertexCode(*reinterpret_cast<QString*>(_v)); break;
        case 13: _t->setSelectedNodeQmlCode(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *NodeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NodeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8NodeViewE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int NodeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 27;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void NodeView::nodesModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void NodeView::arrowsModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void NodeView::activeArrowStartPointChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void NodeView::activeArrowEndPointChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void NodeView::activeArrowEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void NodeView::activeNodesListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void NodeView::selectedNodeNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void NodeView::selectedNodeDescriptionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void NodeView::selectedNodeFragmentCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void NodeView::selectedNodeVertexCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void NodeView::selectedNodeQmlCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void NodeView::nodeGraphCompleteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void NodeView::effectNodeSelectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void NodeView::mainNodeSelectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void NodeView::selectedNodeIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void NodeView::codeSelectorModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void NodeView::codeSelectorIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void NodeView::doubleClickNode()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void NodeView::openDeleteNodeDialog()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void NodeView::openNodeContextMenu()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}
QT_WARNING_POP
