// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#ifndef PROPERTYHANDLER_H
#define PROPERTYHANDLER_H

#include <QQmlPropertyMap>

// This will be used for binding dynamic properties
// changes between C++ and QML.
extern QQmlPropertyMap g_propertyData;

#endif // PROPERTYHANDLER_H
