/****************************************************************************
** Meta object code from reading C++ file 'qsbinspectorhelper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtquickeffectmaker/tools/qqem/qsbinspectorhelper.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsbinspectorhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QsbShaderDataE_t {};
} // unnamed namespace

template <> constexpr inline auto QsbShaderData::qt_create_metaobjectdata<qt_meta_tag_ZN13QsbShaderDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QsbShaderData",
        "currentFile",
        "stage",
        "reflectionInfo",
        "qsbVersion",
        "shaderCount",
        "size"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'currentFile'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'stage'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'reflectionInfo'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'qsbVersion'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'shaderCount'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'size'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QsbShaderData, qt_meta_tag_ZN13QsbShaderDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QsbShaderData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QsbShaderDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QsbShaderDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QsbShaderDataE_t>.metaTypes,
    nullptr
} };

void QsbShaderData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QsbShaderData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->m_currentFile; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->m_stage; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->m_reflectionInfo; break;
        case 3: *reinterpret_cast<int*>(_v) = _t->m_qsbVersion; break;
        case 4: *reinterpret_cast<int*>(_v) = _t->m_shaderCount; break;
        case 5: *reinterpret_cast<int*>(_v) = _t->m_size; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->m_currentFile, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->m_stage, *reinterpret_cast<QString*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->m_reflectionInfo, *reinterpret_cast<QString*>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->m_qsbVersion, *reinterpret_cast<int*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->m_shaderCount, *reinterpret_cast<int*>(_v)); break;
        case 5: QtMocHelpers::setProperty(_t->m_size, *reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN18ShaderSelectorDataE_t {};
} // unnamed namespace

template <> constexpr inline auto ShaderSelectorData::qt_create_metaobjectdata<qt_meta_tag_ZN18ShaderSelectorDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ShaderSelectorData",
        "name",
        "sourceIndex"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'sourceIndex'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ShaderSelectorData, qt_meta_tag_ZN18ShaderSelectorDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ShaderSelectorData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ShaderSelectorDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ShaderSelectorDataE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18ShaderSelectorDataE_t>.metaTypes,
    nullptr
} };

void ShaderSelectorData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ShaderSelectorData *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->m_name; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->m_sourceIndex; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->m_name, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->m_sourceIndex, *reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN18QsbInspectorHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto QsbInspectorHelper::qt_create_metaobjectdata<qt_meta_tag_ZN18QsbInspectorHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QsbInspectorHelper",
        "shaderDataChanged",
        "",
        "sourceSelectorModelChanged",
        "currentSourceIndexChanged",
        "currentSourceCodeChanged",
        "loadQsb",
        "filename",
        "setCurrentSourceIndex",
        "index",
        "shaderData",
        "QsbShaderData",
        "sourceSelectorModel",
        "QVariantList",
        "currentSourceIndex",
        "currentSourceCode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'shaderDataChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceSelectorModelChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentSourceIndexChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentSourceCodeChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadQsb'
        QtMocHelpers::SlotData<bool(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'setCurrentSourceIndex'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'shaderData'
        QtMocHelpers::PropertyData<QsbShaderData>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'sourceSelectorModel'
        QtMocHelpers::PropertyData<QVariantList>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'currentSourceIndex'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'currentSourceCode'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QsbInspectorHelper, qt_meta_tag_ZN18QsbInspectorHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QsbInspectorHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QsbInspectorHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QsbInspectorHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QsbInspectorHelperE_t>.metaTypes,
    nullptr
} };

void QsbInspectorHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QsbInspectorHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->shaderDataChanged(); break;
        case 1: _t->sourceSelectorModelChanged(); break;
        case 2: _t->currentSourceIndexChanged(); break;
        case 3: _t->currentSourceCodeChanged(); break;
        case 4: { bool _r = _t->loadQsb((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->setCurrentSourceIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QsbInspectorHelper::*)()>(_a, &QsbInspectorHelper::shaderDataChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QsbInspectorHelper::*)()>(_a, &QsbInspectorHelper::sourceSelectorModelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QsbInspectorHelper::*)()>(_a, &QsbInspectorHelper::currentSourceIndexChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QsbInspectorHelper::*)()>(_a, &QsbInspectorHelper::currentSourceCodeChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QsbShaderData >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QsbShaderData*>(_v) = _t->shaderData(); break;
        case 1: *reinterpret_cast<QVariantList*>(_v) = _t->sourceSelectorModel(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->currentSourceIndex(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->currentSourceCode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setCurrentSourceIndex(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QsbInspectorHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QsbInspectorHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QsbInspectorHelperE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QsbInspectorHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QsbInspectorHelper::shaderDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QsbInspectorHelper::sourceSelectorModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QsbInspectorHelper::currentSourceIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QsbInspectorHelper::currentSourceCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
